<?php
defined('ABSPATH') or die;

class ThemeContentImport {

    /**
     * Action on admin_notices
     *
     * @global TGM_Plugin_Activation $tgmpa
     */
    public static function contentImportNoticeAction() {
        global $tgmpa;
        $installation_required = !defined('NICEPAGE_PLUGIN_VERSION');
        $import_href = $installation_required
            ? add_query_arg(array('np-import' => '1'), $tgmpa->get_tgmpa_url())
            : '#';
        ?>
        <div id="content-import-notice" class="updated">
            <p>
                <?php echo __('Do you want to import Content?', 'evo2wp1'); ?>
                &nbsp; &nbsp; &nbsp; &nbsp;
                <a id="import-import-content" class="import-button button" href="<?php echo $import_href; ?>"><?php echo __('Import content', 'evo2wp1'); ?></a>
                <a id="import-replace-content" class="<?php if (get_option('np_imported_content') === false) echo 'hidden'; ?> import-button button" href="<?php echo $import_href; ?>"><?php echo __('Replace previously imported content', 'evo2wp1'); ?></a>
                <a id="import-hide-notice" class="import-button button" href="#"><?php echo __('Hide notice', 'evo2wp1'); ?></a>
            </p>
        </div>
        <style>
            .import-button {
                text-decoration: none;
            }
            .import-button.importing:before {
                content: '';
                background-image: url('<?php echo NICEPAGE_PLUGIN_URL; ?>/importer/assets/images/preloader-01.gif');
                display: inline-block;
                width: 13px;
                height: 13px;
                background-size: 100% 100%;
                margin-right: 5px;
            }
        </style>
        <script>
            jQuery(document).ready(function ($) {
                function doAjax(action) {
                    return $.ajax({
                        url: '<?php echo admin_url("admin-ajax.php"); ?>',
                        type: 'GET',
                        data: ({
                            action: action,
                            _ajax_nonce: '<?php echo wp_create_nonce('theme-content-importer'); ?>'
                        })
                    });
                }
                function bindImportAction(action, btn) {
                    var successMsg = <?php echo json_encode(__('Content was successfully imported.', 'evo2wp1')); ?>;
                    var failMsg = <?php echo json_encode(__('An error occurred while importing.', 'evo2wp1')); ?>;

                    btn.unbind("click").click(function() {
                        $(this).addClass('importing');
                        doAjax(action).done(function () {
                            $('#content-import-notice')
                                .html('<p>' + successMsg + '</p>');
                        }).fail(function () {
                            $('#content-import-notice')
                                .removeClass('updated').addClass('error')
                                .html('<p>' + failMsg + '</p>');
                        });
                    });
                }
                <?php if (!$installation_required): ?>
                bindImportAction('theme_import_content', $('#import-import-content'));
                bindImportAction('theme_replace_content', $('#import-replace-content'));
                <?php endif; ?>

                $('#import-hide-notice').unbind("click").click(function() {
                    $('#content-import-notice').remove();
                    doAjax('theme_hide_import_notice');
                });
            });
        </script>
        <?php
    }

    /**
     * Action on admin_notices
     *
     * @global TGM_Plugin_Activation $tgmpa
     */
    public static function pluginRequiredNoticeAction() {
        global $tgmpa;
        ?>
        <div id="content-import-notice" class="notice notice-warning">
            <p><?php _e('Import process required Nicepage plugin to be active.', 'evo2wp1'); ?></p>
            <p><?php $tgmpa->is_plugin_installed('nicepage')
                    ? _e('Please activate Nicepage to continue.', 'evo2wp1')
                    : _e('Please install and activate Nicepage to continue.', 'evo2wp1'); ?></p>
        </div>
        <?php
    }

    /**
     * Action on init
     */
    public static function addImportNoticeAction() {
        remove_action('admin_notices', 'themler_content_import_notice');
        remove_action('admin_notices', 'NpImportNotice::contentImportNoticeAction');

        if (!file_exists(get_template_directory() . '/content/content.json')) {
            return;
        }

        if (!empty($_GET['np-import'])) {
            add_action('admin_notices', 'ThemeContentImport::pluginRequiredNoticeAction');
        } else if (!self::getImportNoticeOption() && (!isset($_GET['page']) || $_GET['page'] !== 'tgmpa-install-plugins')) {
            add_action('admin_notices', 'ThemeContentImport::contentImportNoticeAction');
        }
    }

    public static function addImportNoticeOption() {
        update_option('themler_hide_import_notice', true);
    }

    public static function getImportNoticeOption() {
        return get_option('themler_hide_import_notice');
    }

    public static function removeImportNoticeOption() {
        delete_option('themler_hide_import_notice');
    }

    /**
     * Action on wp_ajax_theme_hide_import_notice
     */
    public static function hideImportNoticeAction() {
        check_ajax_referer('theme-content-importer');
        self::addImportNoticeOption();
    }

    /**
     * Action on wp_ajax_theme_import_content
     */
    public static function importContentAction() {
        check_ajax_referer('theme-content-importer');
        self::_importData(false);
    }

    /**
     * Action on wp_ajax_theme_replace_content
     */
    function replaceContentAction() {
        check_ajax_referer('theme-content-importer');
        self::_importData(true);
    }

    private static function _importData($remove_prev = false) {
        $content_dir = get_template_directory() . '/content';
        self::addImportNoticeOption();

        do_action('nicepage_import_content', $content_dir, $remove_prev);
    }
}

add_action('init', 'ThemeContentImport::addImportNoticeAction', 100); // after after_switch_theme executing
add_action('wp_ajax_theme_hide_import_notice', 'ThemeContentImport::hideImportNoticeAction', 9);
add_action('wp_ajax_theme_import_content', 'ThemeContentImport::importContentAction', 9);
add_action('wp_ajax_theme_replace_content', 'ThemeContentImport::replaceContentAction', 9);

// enable import content banner after switch theme
add_action('after_switch_theme', 'ThemeContentImport::removeImportNoticeOption');