<?php
defined('ABSPATH') or die;

class NpAdminActions {

    /**
     * Filter on get_edit_post_link
     * Add domain parameter for debug
     *
     * @param string $link
     *
     * @return string
     */
    public static function editPostLinkFilter($link) {
        $domain = NpEditor::getDomain();
        if ($domain) {
            $link = add_query_arg(array('domain' => urlencode($domain)), $link);
        }
        return $link;
    }

    /**
     * Action on edit_form_top
     * Add domain hidden field to editor form
     */
    public static function editFormTopFilter() {
        $domain = NpEditor::getDomain();
        if ($domain) {
            printf('<input type="hidden" name="domain" value="%s" />', $domain);
        }
    }

    /**
     * Action on themler_edit_form_buttons
     * Add "Edit with Nicepage" button
     *
     * @param WP_Post $post
     */
    public static function addNicepageButtonAction($post) {
        if (!NpEditor::isAllowedForEditor($post)) {
            return;
        }
        if (np_data_provider($post->ID)->isConvertRequired()) {
?>
            <a href="#" id="convert-in-nicepage" class="button nicepage-editor"><?php _e('Turn to Nicepage', 'nicepage'); ?></a>
<?php
        } else {
?>
            <a href="#" id="edit-in-nicepage" class="button nicepage-editor"><?php _e('Edit with Nicepage', 'nicepage'); ?></a>
<?php
        }
    }

    /**
     * Action on admin_menu
     * Add Nicepage menus and register page capabilities
     */
    public static function addEditorPageAction() {
        $capability = 'edit_pages';
        $menu_slug = 'np_app';
        add_theme_page(__('Nicepage', 'nicepage'), __('Nicepage', 'nicepage'), $capability, $menu_slug, 'np_start');
        add_theme_page(__('Nicepage', 'nicepage'), __('Nicepage', 'nicepage'), $capability, 'np_editor', 'NpAdminActions::editorAction');

        add_submenu_page($menu_slug, __('Import', 'nicepage'), __('Import', 'nicepage'), $capability, 'np_import', 'NpImport::importPage');
        add_submenu_page($menu_slug, __('Settings', 'nicepage'), __('Settings', 'nicepage'), $capability, 'np_settings', 'NpSettings::settingsPage');

        // remove submenu from Appearance
        global $submenu;
        if (isset($submenu['themes.php']) && is_array($submenu['themes.php'])) {
            foreach ($submenu['themes.php'] as $key => $value) {
                if (in_array('np_editor', $value) || in_array('np_app', $value)) {
                    unset($submenu['themes.php'][$key]);
                }
            }
        }
    }

    /**
     * Action on custom_menu_order
     * Add Nicepage menus and register page capabilities
     *
     * @return bool
     */
    public static function addSubmenusAction() {
        global $submenu;
        $menu_slug = NpAdminActions::adminUrlFilter(admin_url('post-new.php?post_type=page&np_new=1'));
        $capability = 'edit_pages';
        $submenu[$menu_slug][10] = array(__('New Page', 'nicepage'), $capability, $menu_slug);
        $submenu[$menu_slug][11] = array(__('Settings', 'nicepage'), $capability, admin_url('admin.php?page=np_settings'));
        $submenu[$menu_slug][12] = array(__('Import', 'nicepage'), $capability, admin_url('admin.php?page=np_import'));
        return true;
    }

    /**
     * Action on
     *  _network_admin_menu,
     *  _user_admin_menu,
     *  _admin_menu
     *
     * Add Nicepage menus and register page capabilities
     */
    public static function addMenuAction() {
        global $menu;
        $menu_slug = NpAdminActions::adminUrlFilter(admin_url('post-new.php?post_type=page&np_new=1'));
        $capability = 'edit_pages';
        $menu['57'] = array(__('Nicepage', 'nicepage'), $capability, $menu_slug, '', 'menu-top menu-icon-nicepage', 'menu-nicepage', 'div');
    }

    public static $thumbnailScale = 0.3;

    /**
     * Action on admin_print_styles
     */
    public static function printMenuStylesAction() {
        ?>
        <style>
            #adminmenu .menu-icon-nicepage div.wp-menu-image {
                background-image: url('<?php echo NICEPAGE_PLUGIN_URL; ?>editor/assets/images/menu-icon.png');
                background-position:50% 55%;
                background-repeat: no-repeat;
                background-size: 16px;
            }

            #adminmenu .menu-icon-nicepage:hover div.wp-menu-image,
            #adminmenu .menu-icon-nicepage.wp-has-current-submenu div.wp-menu-image,
            #adminmenu .menu-icon-nicepage.current div.wp-menu-image {
                opacity: 0.85;
            }

            .nicepage-editor:before {
                background-image: url('<?php echo NICEPAGE_PLUGIN_URL; ?>editor/assets/images/menu-icon.png');
                display: inline-block;
                margin-right: 5px;
                content: "";
                background-size: 16px;
                width: 16px;
                height: 16px;
                background-repeat: no-repeat;
                vertical-align: middle;
                margin-bottom: 3px;
            }

            #nicepage-preview-frame {
                transform: scale(<?php echo self::$thumbnailScale; ?>);
                transform-origin: 0 0;
                height: <?php echo 100 / self::$thumbnailScale; ?>%;
            }

            #nicepage-preview {
                margin-left: auto;
                margin-right: auto;
                cursor: pointer;
                overflow: hidden;
                height: 0;
            }

            #nicepage-preview-frame {
                pointer-events: none;
            }
            #np-loader {
                position: absolute;
                visibility: hidden;
                width: 1800px;
            }
        </style>
        <?php
    }


    /**
     * Get Nicepage editor iframe html
     *
     * @param array $args
     *
     * @return string
     */
    public static function getEditorContainerHtml($args = array()) {
        ob_start();
?>
        <div id="nicepage-editor-container">
            <style>
                html {
                    overflow: hidden !important;
                }
                #wpcontent {
                    padding-left: 0 !important;
                }
                #nicepage-editor-frame {
                    width: 100%;
                    height: calc(100vh - 32px); /*for admin bar*/
                }
            </style>
            <script>
                jQuery(document).scroll(function() {
                    if (jQuery('#nicepage-editor-container').length) {
                        jQuery(this).scrollTop(0);
                    }
                });
            </script>
            <iframe id="nicepage-editor-frame" src="<?php echo NpEditor::getAppLink($args); ?>"></iframe>
        </div>
<?php
        return ob_get_clean();
    }

    /**
     * Action on load-appearance_page_np_editor
     * Print Nicepage editor iframe
     *
     * @deprecated
     */
    public static function editorAction() {
        throw new Exception("Deprecated usage");
    }

    /**
     * Print Nicepage editor page
     */
    public static function appAction() {
        include_once dirname(__FILE__) . '/nicepage-editor.php';
        die();
    }

    /**
     * Action on edit_form_after_title
     * Print page preview (first 4 sections)
     *
     * @param WP_Post $post
     */
    public static function showScreenShotsAction($post) {
        $data_provider = np_data_provider($post->ID);
        if ($data_provider->isNicepage()) {
            ob_start();
?>
            <head>
                <link rel="stylesheet" type="text/css" media="all" href="<?php echo NICEPAGE_PLUGIN_URL; ?>assets/css/nicepage.css">

                <?php echo $data_provider->getPageFonts(); ?>
                <style>
                    <?php echo $data_provider->getStyleCss(); ?>
                    <?php echo $data_provider->getPageHead(); ?>
                </style>
            </head>
<?php
            $frame_head = ob_get_clean();
            ob_start();
?>
            <body class="<?php echo $data_provider->getPageBodyClass(); ?>">
                <?php
                    self::$_previewSectionIdx = 0;
                    echo preg_replace_callback(
                        '#<section [\s\S]*?</section>#',
                        'NpAdminActions::_screenshotSectionsReplacer',
                        $data_provider->getPagePublishHtml()
                    );
                ?>
            </body>
<?php
            $frame_body = ob_get_clean();
?>
            <div id="nicepage-preview">
                <iframe id="nicepage-preview-frame">
                </iframe>
            </div>
            <style>
                #postdivrich {
                    display: none;
                }
            </style>
            <script>
                jQuery(function ($) {
                    var doc = $('#nicepage-preview-frame')[0].contentDocument;
                    doc.open();
                    doc.write("<!DOCTYPE html>\n<html>\n" + <?php echo wp_json_encode($frame_head); ?> + <?php echo wp_json_encode($frame_body); ?> + "</html>");
                    doc.close();

                    $("#nicepage-preview-frame").load(function() {
                        var totalHeight = 0,
                            width = 1600;

                        $('#nicepage-preview-frame').css('width', width + 'px');
                        $('#nicepage-preview').css('width', width * <?php echo self::$thumbnailScale; ?> + 'px');
                        $('#nicepage-preview-frame').contents().find('section').each(function () {
                            totalHeight += $(this).height();
                        });
                        $('#nicepage-preview').css('height', totalHeight * <?php echo self::$thumbnailScale; ?>);
                    });
                });
            </script>
<?php
        }
    }

    /**
     * @param array $m
     *
     * @return string
     */
    public static function _screenshotSectionsReplacer($m) {
        if (self::$_previewSectionIdx >= 4) {
            return '';
        }
        self::$_previewSectionIdx++;
        return $m[0];
    }

    private static $_previewSectionIdx;


    /**
     * Action on admin_head
     * Print page editor scripts and styles
     */
    public static function printPreviewStylesAction() {
        global $post;

        if ($post && $post->ID) {
            if (!NpEditor::isAllowedForEditor($post)) {
                return;
            }
            np_data_provider($post->ID, true)->clear();
        }
?>
        <style>
            #nicepage-preview {
                font-size: 0;
                line-height: 0;
            }
            #nicepage-preview a img {
                max-width: 100%;
            }
        </style>

        <script>
            var menuFolded;

            function collapseMenu() {
                jQuery('body').addClass('folded');
                jQuery(document).trigger('wp-collapse-menu', {state: 'folded'});
            }
            function startEditor() {
                menuFolded = jQuery('body').hasClass('folded');
                jQuery(window).scrollTop(0);
                var editorContainer = <?php echo wp_json_encode(self::getEditorContainerHtml(array('post_id' => $post ? $post->ID : 0))); ?>;
                jQuery('#wpbody-content').prepend(editorContainer);
                collapseMenu();
            }
            function closeEditor(data) {
                if (data.needRefresh) {
                    if (!data.closeUrl || window.location.href === data.closeUrl) {
                        window.location.reload();
                    } else {
                        window.location.href = data.closeUrl;
                    }
                } else {
                    jQuery('#nicepage-editor-container').remove();
                    jQuery('body').toggleClass('folded', menuFolded);
                }
            }

            function createAnchorListItem(anchor) {
                if (!anchor) {
                    return anchor;
                }
                var result = jQuery('<li><input type="hidden" class="item-permalink"><span class="item-title"></span><span class="item-info">Section</span></li>');
                result.children('input').attr('value', anchor.url);
                result.children('span.item-title').text(anchor.title);
                return result;
            }

            function addListItem(table, element) {
                table.find('ul').append(element);
            }

            function fixAlternateStyles(table) {
                table.find('li:even').addClass('alternate');
                table.find('li:odd').removeClass('alternate');
            }

            function addAnchorLinks(table, data) {
                if (!data || !data.length) {
                    return;
                }
                var listItems = jQuery.map(data, createAnchorListItem);
                jQuery.each(listItems, function (i, item) {
                    addListItem(table, item);
                });
                fixAlternateStyles(table);
            }

            function selectListItemByUrl(list, url) {
                var matchUrl = function (index, element) {
                    return jQuery(element).children('.item-permalink').prop('value') === url;
                }
                var listItems = list.find('li'),
                    active = listItems.filter(matchUrl);
                if (active.hasClass('selected')) {
                    return;
                }
                listItems.removeClass('selected');
                active.addClass('selected');
            }

            function handleAnchorListItemClick(event) {
                var element = jQuery(this),
                    dialog = element.closest('#wp-link'),
                    list = element.closest('ul');
                wpLink.updateFields(event, element);
                selectListItemByUrl(list, jQuery('#wp-link-url').prop('value'));
            }

            function handleAnchorLinkCheckboxChange(event) {
                var input = jQuery(this),
                    dialog = input.closest('#wp-link'),
                    list = dialog.find('#anchor-links-list'),
                    value = input.attr('value'),
                    searchField = dialog.find('.link-search-wrapper label');
                if (value === "section") {
                    list.removeClass('hidden');
                    searchField.addClass('hidden');
                    selectListItemByUrl(list, jQuery('#wp-link-url').prop('value'));
                } else {
                    list.addClass('hidden');
                    searchField.removeClass('hidden');
                }
            }

            function markAnchorByUrl(dialog, url) {
                if (url && url.indexOf('#') === 0 && url !== '#') {
                    dialog.find('.anchor-link input[value=section]').click();
                } else {
                    dialog.find('.anchor-link input[value=page]').click();
                }
            }

            function addAnchorsToLinkDialog(dialog, data) {
                if (!data || !data.l || !data.l.anchorLink || !data.anchorsList || !data.anchorsList.length) {
                    return;
                }
                dialog.find('.link-target').after(
                    '<div class="anchor-link" style="margin-top: 5px"><label><span style="vertical-align: bottom">' +
                    data.l.Destination + 
                    '</span>\n<input type="radio" name="link-destination" value="page"> ' +
                    data.l.pageLink +
                    '</label><br><label><span></span>\n<input type="radio" name="link-destination" value="section"> ' +
                    data.l.anchorLink +
                    '</label></div>'
                );
                dialog.find('#search-panel').append(
                    '<div id="anchor-links-list" class="query-results hidden" tabindex="0"><ul></ul>' +
                    '<div class="river-waiting"><span class="spinner"></span></div></div>'
                );
                dialog.find('#anchor-links-list').on('click', 'li', handleAnchorListItemClick);
                dialog.find('.anchor-link input').on('change', handleAnchorLinkCheckboxChange);
                addAnchorLinks(dialog.find('#anchor-links-list'), data.anchorsList);
                markAnchorByUrl(dialog, data.url);
            }

            function cleanupLinkDialog(dialog) {
                dialog.find('.anchor-link input').off('change', handleAnchorLinkCheckboxChange);
                dialog.find('#anchor-links-list').off('click', 'li', handleAnchorListItemClick);
                dialog.find('.anchor-link, #anchor-links-list').remove();
            }

            function postMessageListener(event) {
                if (event.origin !== location.origin) {
                    return;
                }
                var data = JSON.parse(event.data);
                if (!data) {
                    return;
                }
                var childWindow = jQuery('#nicepage-editor-frame')[0].contentWindow;

                if (data.action === 'close') {
                    closeEditor(data);
                } else if (data.action === 'editLinkDialogOpen') {
                    wpLink.open();
                    jQuery('.wp-link-text-field, #wplink-link-existing-content').remove();

                    jQuery('#wp-link-target').prop('checked', !!data.data.blank);
                    jQuery('#wp-link-url').prop('value', data.data.url);
                                        
                    addAnchorsToLinkDialog(jQuery('#wp-link'), data.data);

                    var wpLinkUpdate = wpLink.update;
                    wpLink.update = function () {
                        var attrs = wpLink.getAttrs();
                        childWindow.postMessage(JSON.stringify({
                            action: 'editLinkDialogClose',
                            data: {
                                url: attrs.href,
                                blank: !!attrs.target
                            }
                        }), window.location.origin);
                        wpLink.update = wpLinkUpdate;
                        wpLink.close();
                    };

                    jQuery(document).one('wplink-close', function () {
                        cleanupLinkDialog(jQuery('#wp-link'));
                        childWindow.postMessage(JSON.stringify({action: 'editLinkDialogClose'}), window.location.origin);
                    });
                }
            }
            jQuery(function ($) {
                menuFolded = $('body').hasClass('folded');

                $('#edit-in-nicepage, #convert-in-nicepage, #nicepage-preview').click(startEditor);

                if (location.search.indexOf('np_new=') !== -1 || location.search.indexOf('np_edit=') !== -1) {
                    collapseMenu();
                }

                <?php if (_arr($_GET, 'np_new')) : ?>
                $('.wp-menu-open').removeClass('wp-has-current-submenu wp-menu-open').addClass('wp-not-current-submenu');
                $('#menu-nicepage, #menu-nicepage > a').removeClass('wp-not-current-submenu').addClass('wp-has-current-submenu wp-menu-open');
                <?php endif; ?>
            });

            if (window.addEventListener) {
                window.addEventListener("message", postMessageListener);
            } else {
                window.attachEvent("onmessage", postMessageListener); // IE8
            }

        </script>
<?php
    }


    /**
     * Action on admin_head
     * Print prepared data for Nicepage-editor
     *
     * @throws Exception
     */
    public static function printBridgeAction() {
        global $post;
        if (!$post || !NpEditor::isAllowedForEditor($post)) {
            return;
        }
        $user = wp_get_current_user();
        $uid = (int)$user->ID;
        $ajax_nonce = wp_create_nonce('np-upload');
        $edit_post_url = NpAdminActions::adminUrlFilter(admin_url('post.php?action=edit')) . '&post={id}';

        $settings = array(
            'actions' => array(
                'uploadImage'            => NpAction::getActionUrl('np_upload_image'),
                'savePage'               => NpAction::getActionUrl('np_save_page'),
                'saveSiteSettings'       => NpAction::getActionUrl('np_save_site_settings'),
                'getSite'                => NpAction::getActionUrl('np_get_site'),
                'getSitePosts'           => NpAction::getActionUrl('np_get_site_posts'),
            ),
            'ajaxData' => array(
                'uid' => $uid,
                '_ajax_nonce' => $ajax_nonce,
            ),
            'uploadImageOptions' => array(
                'formFileName' => 'async-upload',
                'params' => array(
                    'html-upload' => 'Upload',
                    '_wpnonce' => wp_create_nonce('media-form'),
                    'uid' => $uid,
                    '_ajax_nonce' => $ajax_nonce,
                ),
            ),
            'editPostUrl' => $edit_post_url,
            'dashboardUrl' => NpAdminActions::adminUrlFilter(admin_url()),
            'loginUrl' => wp_login_url($edit_post_url),
            'pageId' => np_data_provider($post->ID)->isConvertRequired() ? '' : $post->ID,
            'startPageId' => $post->ID,
            'startPageTitle' => $post->post_title,
        );

        $cms_settings = array(
            'defaultImageUrl' => NpAttachments::getDefaultImageUrl(),
            'isFirstStart' => !!_arr($_GET, 'np_new')
        );

        $data_provider = np_data_provider($post->ID);

        $page_obj = NpAction::getPost($post);
        $data = array(
            'site' => NpGetSiteAction::getSite(),
            'pageHtml' => $data_provider->getPageHtml(),
            'htmlUrl' => add_query_arg(array('np_html' => '1', 'preview' => 'true'), get_permalink($post->ID)),
            'nicePageCss' => file_get_contents(NICEPAGE_PLUGIN_PATH . 'assets/css/nicepage-dynamic.css'),
        );
        $page_found = false;
        foreach ($data['site']['items'] as $page) {
            if ($page['id'] === $page_obj['id']) {
                $page_found = true;
                break;
            }
        }
        if (!$page_found) {
            array_unshift($data['site']['items'], $page_obj);
        }

        $data['info'] = array(
            'taxonomies' => array(),
            'menus' => array(),
            'productsExists' => class_exists('WooCommerce'),
            'newPageUrl' => NpAdminActions::adminUrlFilter(admin_url('post-new.php?post_type=page&np_new=1')),
            'forceModified' => !get_post_meta($post->ID, '_np_site_style_css_used_ids', true),
        );
        foreach (get_taxonomies(array('show_tagcloud' => true), 'object') as $taxonomy) {
            $data['info']['taxonomies'][] = array('name' => $taxonomy->name, 'label' => $taxonomy->label);
        }
        foreach (wp_get_nav_menus() as $menu) {
            $data['info']['menus'][] = array('id' => $menu->term_id . '', 'label' => $menu->name);
        }
?>
        <script>
            var dataBridgeData = <?php echo wp_json_encode($data, JSON_PRETTY_PRINT); ?>,
                callbacks = [],
                attemptsCount = 0;

            window.dataBridge = {
                getSite: function getSite() {
                    return dataBridgeData.site;
                },
                getPageHtml: function getPageHtml() {
                    return dataBridgeData.pageHtml;
                },
                getNicePageCss: function getNicePageCss() {
                    return dataBridgeData.nicePageCss;
                },
                getStartTerm: function getStartTerm() {
                    return "<?php echo $data_provider->isConvertRequired() ? 'site:wordpress:' . $post->ID : ''; ?>";
                },
                getInfo: function getInfo() {
                    return dataBridgeData.info;
                },
                doLoggedIn: function doLoggedIn(func) {
                    callbacks.push(func);

                    // show login dialog
                    if (jQuery('#wp-auth-check-wrap').hasClass('hidden')) {
                        if (attemptsCount > 0) {
                            // login dialog will not be shown a second time
                            // see  "$(document).off( 'heartbeat-tick.wp-auth-check' );" in wp-auth-check.js
                            callbacks.forEach(function (cb) {
                                cb(false);
                            });
                            callbacks = [];
                            return;
                        }
                        jQuery(document).trigger('heartbeat-tick', [{'wp-auth-check': false}]);
                    }
                },
                settings: <?php echo wp_json_encode($settings, JSON_PRETTY_PRINT); ?>,
                cmsSettings: <?php echo wp_json_encode($cms_settings, JSON_PRETTY_PRINT); ?>
            };

            jQuery(document).on("heartbeat-tick.wp-refresh-nonces", function(c, d) {
                if (d['wp-refresh-post-nonces'] && typeof dataBridge === 'object') {
                    dataBridge.settings.ajaxData._ajax_nonce = d['wp-refresh-post-nonces'].replace.np_ajax_nonce;
                    dataBridge.settings.uploadImageOptions.params._ajax_nonce = d['wp-refresh-post-nonces'].replace.np_ajax_nonce;
                    dataBridge.settings.uploadImageOptions.params._wpnonce = d['wp-refresh-post-nonces'].replace.np_upload_image_nonce;

                    attemptsCount++;
                    callbacks.forEach(function (cb) {
                        cb(true);
                    });
                    callbacks = [];
                }
            });
            jQuery(document).on('click', '#wp-auth-check-wrap .wp-auth-check-close', function() {
                attemptsCount++;
                callbacks.forEach(function (cb) {
                    cb(false);
                });
            });



        </script>
<?php
    }

    /**
     * Action on in_admin_header
     * Prepare theme typography scripts
     */
    public static function printLoaderFrameScripts() {
        global $post;
        if (!$post || !NpEditor::isAllowedForEditor($post)) {
            return;
        }

        $defined_settings = apply_filters('nicepage_theme_settings', array());
        if ($defined_settings) {
?>
            <script>
                dataBridgeData.info.themeTypography = <?php echo wp_json_encode($defined_settings['typography']); ?>;
                dataBridgeData.info.themeFontScheme = <?php echo wp_json_encode($defined_settings['fontScheme']); ?>;
                dataBridgeData.info.themeColorScheme = <?php echo wp_json_encode($defined_settings['colorScheme']); ?>;
            </script>
<?php
            return;
        }

?>
        <script src="<?php echo NICEPAGE_PLUGIN_URL; ?>/editor/assets/js/typography-parser.js?v=<?php echo NICEPAGE_PLUGIN_VERSION; ?>"></script>
        <iframe id="np-loader"></iframe>

        <script>
            var loadCallback;

            var needResetCache = !localStorage.np_theme_typography_cache ||
                document.cookie.indexOf('np_theme_typography_cache_force_refresh=1') !== -1;

            if (needResetCache) {
                delete localStorage.np_theme_typography_cache;
            }

            window.loadAppHook = function (load) {
                if (localStorage.np_theme_typography_cache) {
                    jQuery.extend(dataBridgeData.info, JSON.parse(localStorage.np_theme_typography_cache));
                    console.log('Regular load app.js');
                    load();
                    return;
                }
                loadCallback = load;
            };

            var loaderIframe = document.getElementById('np-loader');
            loaderIframe.addEventListener("load", function() {
                localStorage.np_theme_typography_cache = JSON.stringify(NpTypographyParser.parse(loaderIframe));
                document.cookie = 'np_theme_typography_cache_force_refresh=';
                jQuery(loaderIframe).remove();
                console.log('Typography cache updated');
                jQuery.extend(dataBridgeData.info, JSON.parse(localStorage.np_theme_typography_cache));
                if (loadCallback) {
                    console.log('Deferred load app.js');
                    loadCallback();
                }
            });
            loaderIframe.src = dataBridgeData.htmlUrl;
        </script>
<?php
    }

    /**
     * Filter on the_posts
     * Create fake post for testing typography
     *
     * @param array $posts
     *
     * @return stdClass[]
     */
    public static function fakePostsFilter($posts) {
        global $wp_query;

        $post = new stdClass;
        $post->ID = -1;
        $post->post_author = 1;
        $post->post_date = current_time('mysql');
        $post->post_date_gmt = current_time('mysql', 1);
        $post->post_content = '<div id="np-test-container"></div>';
        $post->post_title = 'TEST';
        $post->post_excerpt = '';
        $post->post_status = 'publish';
        $post->comment_status = 'closed';
        $post->ping_status = 'closed';
        $post->post_password = '';
        $post->post_name = 'test-theme-page';
        $post->to_ping = '';
        $post->pinged = '';
        $post->modified = $post->post_date;
        $post->modified_gmt = $post->post_date_gmt;
        $post->post_content_filtered = '';
        $post->post_parent = 0;
        $post->guid = get_home_url('/' . $post->post_name);
        $post->menu_order = 0;
        $post->post_tyle = 'page';
        $post->post_mime_type = '';
        $post->comment_count = 0;

        $wp_query->is_page = true;
        $wp_query->is_singular = true;
        $wp_query->is_home = false;
        $wp_query->is_archive = false;
        $wp_query->is_category = false;
        unset($wp_query->query['error']);
        $wp_query->query_vars['error'] = '';
        $wp_query->is_404 = false;

        return array($post);
    }


    /**
     * Filter on admin_url
     * Add domain parameter if need
     *
     * @param string $url
     *
     * @return string
     */
    public static function adminUrlFilter($url) {
        if (isset($_GET['domain'])) {
            $url = add_query_arg(array('domain' => urlencode($_GET['domain'])), $url);
        }
        return $url;
    }

    /**
     * Action on in_admin_header
     * Print Nicepage-editor iframe at the top
     */
    public static function printEditorAction() {
        global $post;
        if (_arr($_GET, 'np_new') || _arr($_GET, 'np_edit')) {
            if ($post && $post->ID && !_arr($_GET, 'np_new')) {
                np_data_provider($post->ID, true)->clear();
            }
            echo NpAdminActions::getEditorContainerHtml(array('post_id' => $post ? $post->ID : 0));
        }
    }

    /**
     * Action in init
     */
    public static function init() {
        add_filter('page_row_actions', 'NpAdminActions::pageRowAction');
    }

    /**
     * Action on page_row_actions
     * Add "Edit with Nicepage" links to pages list
     *
     * @param array $actions
     *
     * @return array
     */
    public static function pageRowAction($actions) {
        $post = get_post();
        if (np_data_provider($post->ID)->isConvertRequired()) {
            $actions['edit_in_nicepage'] = '<a href="' . add_query_arg(array('np_edit' => '1'), get_edit_post_link($post->ID)) . '">' . __('Turn to Nicepage', 'nicepage') . '</a>';
        } else {
            $actions['edit_in_nicepage'] = '<a href="' . add_query_arg(array('np_edit' => '1'), get_edit_post_link($post->ID)) . '">' . __('Edit with Nicepage', 'nicepage') . '</a>';
        }
        return $actions;
    }

    /**
     * Action on wp_refresh_nonces
     * Add new nonces to replace them in nicepageSettings
     *
     * @param array  $response
     * @param array  $data
     * @param string $screen_id
     *
     * @return array
     */
    public static function refreshNonsesFilter($response, $data, $screen_id) {
        if (array_key_exists('wp-refresh-post-nonces', $data)) {
            if (!$post_id = absint($data['wp-refresh-post-nonces']['post_id'])) {
                return $response;
            }

            if (!current_user_can('edit_post', $post_id)) {
                return $response;
            }

            $response['wp-refresh-post-nonces']['replace']['np_ajax_nonce'] = wp_create_nonce('np-upload');
            $response['wp-refresh-post-nonces']['replace']['np_upload_image_nonce'] = wp_create_nonce('media-form');
        }

        return $response;
    }

    /**
     * Action on after_switch_theme
     * Set cookie for refreshing typography cache
     */
    public static function forceRefreshTypographyCache() {
        setcookie('np_theme_typography_cache_force_refresh', '1', time() + 3*YEAR_IN_SECONDS);
    }
}

// @codingStandardsIgnoreStart
/**
 * Action on edit_form_after_title
 * NOTE: For backward compatibility with themler-core do not rename this function
 *
 * @param WP_Post $post
 */
function upage_edit_form_buttons($post) {
    ob_start();
    do_action('themler_edit_form_buttons', $post);
    $html = ob_get_clean();

    if ($html) {
        ?>
        <div style="margin-top: 5px; margin-bottom: 10px;">
            <?php echo $html; ?>
        </div>
        <?php
    }
}
// @codingStandardsIgnoreEnd
if (!has_action('edit_form_after_title', 'themler_edit_form_buttons')) {
    add_action('edit_form_after_title', 'upage_edit_form_buttons');
}

add_filter('get_edit_post_link', 'NpAdminActions::editPostLinkFilter');
add_action('edit_form_top', 'NpAdminActions::editFormTopFilter');
add_action('themler_edit_form_buttons', 'NpAdminActions::addNicepageButtonAction');
add_action('admin_menu', 'NpAdminActions::addEditorPageAction');
add_action('_network_admin_menu', 'NpAdminActions::addMenuAction');
add_action('_user_admin_menu', 'NpAdminActions::addMenuAction');
add_action('_admin_menu', 'NpAdminActions::addMenuAction');
add_action('admin_print_styles', 'NpAdminActions::printMenuStylesAction');
add_action('load-appearance_page_np_app', 'NpAdminActions::appAction');
add_action('edit_form_after_title', 'NpAdminActions::showScreenShotsAction', 100);
add_action('admin_head', 'NpAdminActions::printPreviewStylesAction');
add_action('admin_head', 'NpAdminActions::printBridgeAction');
add_action('custom_menu_order', 'NpAdminActions::addSubmenusAction');
add_action('in_admin_header', 'NpAdminActions::printEditorAction');
add_action("in_admin_header", 'NpAdminActions::printLoaderFrameScripts');
add_action('init', 'NpAdminActions::init');

// after wp_refresh_post_nonces
add_filter('wp_refresh_nonces', 'NpAdminActions::refreshNonsesFilter', 11, 3);

add_action("after_switch_theme", 'NpAdminActions::forceRefreshTypographyCache');

if (Nicepage::isHtmlQuery()) {
    add_filter('the_posts', 'NpAdminActions::fakePostsFilter', 1003);
}