<?php
defined('ABSPATH') or die;

require_once dirname(__FILE__) . '/class-np-shortcodes.php';

class Nicepage {

    /**
     * Filter on the_content
     *
     * @param string $content
     *
     * @return string
     */
    public static function theContentFilter($content) {
        $post = get_post();
        if ($post) {
            $sections_html = self::html($post->ID);
            if ($sections_html) {
                $content = $sections_html;
            }
        }
        return $content;
    }

    /**
     * Get processed publishHtml for page
     *
     * @param string|int $post_id
     *
     * @return string
     */
    public static function html($post_id) {
        $sections_html = np_data_provider($post_id)->getPagePublishHtml();
        if ($sections_html) {
            self::$_formIdx = 0;
            self::$_formsSources = NpForms::getPageForms($post_id);
            $sections_html = self::processContent($sections_html);

            if (self::isAutoResponsive($post_id)) {
                $sections_html = self::_getAutoResponsiveScript($post_id) . $sections_html;
            }
            if (!self::isNpTheme()) {
                $sections_html = '<div class="' . implode(' ', self::bodyClassFilter(array())) . '">' . $sections_html . "</div>";
            }
        }

        return $sections_html;
    }

    /**
     * Filter on body_class
     *
     * Add page classes to <body>
     *
     * @param string[] $classes
     *
     * @return string[]
     */
    public static function bodyClassFilter($classes) {
        if (self::isHtmlQuery()) {
            return $classes;
        }

        $post = get_post();
        if ($post) {
            $class = np_data_provider($post->ID)->getPageBodyClass();
            if ($class && is_singular()) {
                $classes[] = $class;

                if (self::isAutoResponsive($post->ID)) {
                    $initial_mode = self::_getInitialResponsiveMode($post->ID);
                    foreach (array_reverse(self::$responsiveModes) as $mode) {
                        $classes[] = self::$responsiveBorders[$mode]['CLASS'];

                        if ($mode === $initial_mode) {
                            break;
                        }
                    }
                }
            }
        }
        return $classes;
    }

    /**
     * Action on wp_footer
     * Print backlink html
     */
    public static function wpFooterAction() {
        $post = get_post();
        if ($post) {
            $backlink = np_data_provider($post->ID)->getPageBacklink();
            if (get_option('np_hide_backlink')) {
                $backlink = str_replace('u-backlink', 'u-backlink u-hidden', $backlink);
            }
            echo $backlink;
        }
    }

    /**
     * Function for publish_html postprocessing
     *
     * @param string $content
     *
     * @return string
     */
    public static function processContent($content) {
        //$content = self::_productLinksFilter($content);
        $content = self::_processForms($content);
        $content = self::_prepareShortcodes($content);
        $content = do_shortcode($content);
        return $content;
    }

    private static $_formIdx;
    private static $_formsSources;

    /**
     * Process forms
     *
     * @param string $content
     *
     * @return string
     */
    private static function _processForms($content) {
        return preg_replace_callback(NpForms::$formRe, 'Nicepage::_processForm', $content);
    }

    /**
     * Convert HTML-placeholders into shortcodes
     *
     * @param string $content
     *
     * @return string
     */
    private static function _prepareShortcodes($content) {
        $content = preg_replace('#<!--(\/?)(position|block|block_header|block_header_content|block_content_content)-->#', '[$1np_$2]', $content);
        return $content;
    }

    /**
     * Process form
     * Callback for preg_replace_callback
     *
     * @param array $match
     *
     * @return string
     */
    private static function _processForm($match) {
        $form_html = $match[0];
        $form_id = isset(self::$_formsSources[self::$_formIdx]['id']) ? self::$_formsSources[self::$_formIdx]['id'] : 0;

        $return = NpForms::getHtml($form_id, $form_html);
        if (self::$_formIdx === 0) {
            $return = NpForms::getScriptsAndStyles() . "\n" . $return;
        }
        self::$_formIdx++;
        return $return;
    }

    /**
     * Process product add-to-cart links
     *
     * @param string $html
     *
     * @return string
     */
    private static function _productLinksFilter($html) {
        if (!function_exists('wc_get_product')) {
            return $html;
        }

        return preg_replace_callback('#<a(.*?)</a>#', 'Nicepage::_productLinksFilterCallback', $html);
    }

    /**
     * _productLinksFilter preg_replace_callback callback
     *
     * @param array $match
     *
     * @return string
     */
    private static function _productLinksFilterCallback($match) {
        if (preg_match('#href="\#product-(\d+)#', $match[1], $m)) {
            $product_id = (int) $m[1];

            if (preg_match('#class="([^"]*)"#', $match[1], $m)) {
                $class = $m[1];
            } else {
                $class = '';
            }
            global $product;
            $prev_product = $product;
            $product = wc_get_product($product_id);
            ob_start();
            woocommerce_template_loop_add_to_cart();
            $product = $prev_product;
            $add_to_cart_html = ob_get_clean();
            $add_to_cart_html = str_replace('class="button', 'class="' . $class, $add_to_cart_html);
            return $add_to_cart_html;
        }
        return $match[0];
    }

    /**
     * Filter on template_include
     *
     * @param string $template_path
     *
     * @return string
     */
    public static function templateFilter($template_path) {
        global $post;
        if ($post && is_singular() && np_data_provider($post->ID)->isNicepage()) {
            $np_template = NpMetaOptions::get($post->ID, 'np_template');
            $np_template = apply_filters('nicepage_template', $np_template, $post->ID, $template_path);
            if ($np_template) {
                $template_path = dirname(__FILE__) . "/../templates/$np_template.php";
            }
        }
        return $template_path;
    }

    /**
     * Check is it query for getting dummy page
     *
     * @return bool
     */
    public static function isHtmlQuery() {
        return !empty($_GET['np_html']);
    }

    /**
     * Action on wp_head
     */
    public static function addHeadStyles() {
        if (self::isHtmlQuery() || !is_singular()) {
            return;
        }

        $post_id = get_the_ID();
        $data_provider = np_data_provider($post_id);

        echo $data_provider->getPageFonts();

        $styles = $data_provider->getPageHead();
        if (self::isAutoResponsive($post_id)) {
            $styles = preg_replace('#\/\*RESPONSIVE_MEDIA\*\/([\s\S]*?)\/\*\/RESPONSIVE_MEDIA\*\/#', '', $styles);
        } else {
            $styles = preg_replace('#\/\*RESPONSIVE_CLASS\*\/([\s\S]*?)\/\*\/RESPONSIVE_CLASS\*\/#', '', $styles);
        }

        echo "<style>\n$styles</style>\n";

        $description = $data_provider->getPageDescription();
        if ($description) {
            echo "<meta name=\"description\" content=\"$description\">\n";
        }

        $keywords = $data_provider->getPageKeywords();
        if ($keywords) {
            echo "<meta name=\"keywords\" content=\"$keywords\">\n";
        }

        $meta_tags = $data_provider->getPageMetaTags();
        if ($meta_tags) {
            echo $meta_tags . "\n";
        }

        $customHeadHtml = $data_provider->getPageCustomHeadHtml();
        if ($customHeadHtml) {
            echo $customHeadHtml . "\n";
        }
    }

    /**
     * Action on wp_head
     */
    public static function addHeadStyles2() {
        if (self::isHtmlQuery()) {
            return;
        }

        $post_id = get_the_ID();
        $data_provider = np_data_provider($post_id);

        if (is_singular() && $data_provider->isNicepage()) {
            $site_style_css = $data_provider->getStyleCss();
            if ($site_style_css) {
                if (self::isNpTheme()) {
                    $site_style_css = preg_replace('#<style.*?(typography|font-scheme|color-scheme)="Theme [\s\S]*?<\/style>#', '', $site_style_css);
                }
                echo "<style>$site_style_css</style>\n";
            }
        }
    }

    /**
     * Check if need to enable auto-responsive
     *
     * @param string|int $page_id
     *
     * @return bool
     */
    public static function isAutoResponsive($page_id) {
        if (self::isNpTheme()) {
            return false;
        }
        if (NpMetaOptions::get($page_id, 'np_template') === 'html') {
            return false;
        }
        return !!NpSettings::getOption('np_auto_responsive');
    }

    /**
     * Filter on single_post_title
     *
     * @param string  $title
     * @param WP_Post $post
     *
     * @return string
     */
    public static function singlePostTitleFilter($title, $post) {
        $post_id = $post->ID;
        $custom_title = np_data_provider($post_id)->getPageTitleInBrowser();
        if ($custom_title) {
            $title = $custom_title;
        }
        return $title;
    }

    /**
     * Action on wp_enqueue_scripts
     */
    public static function addScriptsAndStylesAction() {
        if (self::isHtmlQuery()) {
            return;
        }

        if (NpSettings::getOption('np_include_jquery')) {
            wp_register_script("nicepage-jquery", NICEPAGE_PLUGIN_URL . 'assets/js/jquery.js', array(), NICEPAGE_PLUGIN_VERSION);
            wp_enqueue_script("nicepage-jquery");

            wp_register_script("nicepage-script", NICEPAGE_PLUGIN_URL . 'assets/js/nicepage.js', array('nicepage-jquery'), NICEPAGE_PLUGIN_VERSION);
        } else {
            wp_register_script("nicepage-script", NICEPAGE_PLUGIN_URL . 'assets/js/nicepage.js', array('jquery'), NICEPAGE_PLUGIN_VERSION);
        }

        wp_register_style("nicepage-style", NICEPAGE_PLUGIN_URL . 'assets/css/nicepage.css', array(), NICEPAGE_PLUGIN_VERSION);
        wp_enqueue_style("nicepage-style");

        if (is_singular()) {
            global $post;
            if (self::_getInitialResponsiveMode($post->ID) !== 'XL') {
                wp_register_style("nicepage-responsive", NICEPAGE_PLUGIN_URL . 'assets/css/responsive.css', array('nicepage-style'), NICEPAGE_PLUGIN_VERSION);
                wp_enqueue_style("nicepage-responsive");
            } else {
                wp_register_style("nicepage-media", NICEPAGE_PLUGIN_URL . 'assets/css/media.css', array('nicepage-style'), NICEPAGE_PLUGIN_VERSION);
                wp_enqueue_style("nicepage-media");
            }
        }

        wp_enqueue_script("nicepage-script");
    }

    public static $responsiveModes = array('XS', 'SM', 'MD', 'LG', 'XL');
    public static $responsiveBorders = array(
        'XL' => array(
            'CLASS' => '',
            'MAX' => 1000000,
        ),
        'LG' => array(
            'CLASS' => 'u-lg',
            'MAX' => 1199,
        ),
        'MD' => array(
            'CLASS' => 'u-md',
            'MAX' => 991,
        ),
        'SM' => array(
            'CLASS' => 'u-sm',
            'MAX' => 767,
        ),
        'XS' => array(
            'CLASS' => 'u-xs',
            'MAX' => 575,
        ),
    );

    /**
     * Get initial responsive mode using $GLOBALS['content_width']
     *
     * @param string|int $post_id
     *
     * @return mixed|string
     */
    private static function _getInitialResponsiveMode($post_id) {
        if (!self::isAutoResponsive($post_id)) {
            return 'XL';
        }
        if (NpMetaOptions::get($post_id, 'np_template')) {
            return 'XL';
        }

        global $content_width;
        if (!isset($content_width) || !$content_width) {
            return 'XL';
        }

        $width = (int) $content_width;

        foreach (self::$responsiveModes as $mode) {
            if ($width <= self::$responsiveBorders[$mode]['MAX']) {
                return $mode;
            }
        }
        return 'XL';
    }

    /**
     * Auto-responsive script
     *
     * @param int|string $post_id
     *
     * @return string
     */
    private static function _getAutoResponsiveScript($post_id) {
        ob_start();
        ?>
        <script>
            (function ($) {
                var ResponsiveCms = window.ResponsiveCms;
                if (!ResponsiveCms) {
                    return;
                }
                ResponsiveCms.contentDom = $('script:last').parent();
                ResponsiveCms.prevMode = <?php echo wp_json_encode(self::_getInitialResponsiveMode($post_id)); ?>;
                ResponsiveCms.mediaStyleUrl = <?php echo wp_json_encode(preg_replace('#^https?:#', '', NICEPAGE_PLUGIN_URL . 'assets/css/media.css')); ?>;
                ResponsiveCms.responsiveStyleUrl = <?php echo wp_json_encode(preg_replace('#^https?:#', '', NICEPAGE_PLUGIN_URL . 'assets/css/responsive.css')); ?>;

                if (typeof ResponsiveCms.recalcClasses === 'function') {
                    ResponsiveCms.recalcClasses();
                }
            })(jQuery);
        </script>
        <?php
        return ob_get_clean();
    }

    /**
     * Action on init
     */
    public static function initAction() {
        if (self::isNpTheme()) {
            add_filter('body_class', 'Nicepage::bodyClassFilter');
        }
    }

    /**
     * Check is it Nicepage theme
     *
     * @return bool
     */
    public static function isNpTheme() {
        if (self::$_themeSettings === null) {
            self::$_themeSettings = apply_filters('nicepage_theme_settings', array());
        }
        return !!self::$_themeSettings;
    }
    private static $_themeSettings = null;
}

add_action('init', 'Nicepage::initAction');
add_filter('the_content', 'Nicepage::theContentFilter');
add_action('wp_footer', 'Nicepage::wpFooterAction', 1001);
add_action('wp_enqueue_scripts', 'Nicepage::addScriptsAndStylesAction', 9); // add before theme styles
add_action('wp_head', 'Nicepage::addHeadStyles');
add_action('wp_head', 'Nicepage::addHeadStyles2', 1004);
add_filter('template_include', 'Nicepage::templateFilter');
add_filter('single_post_title', 'Nicepage::singlePostTitleFilter', 9, 2);

if (Nicepage::isHtmlQuery()) {
    show_admin_bar(false);
}